/* $Id: atari_dc.c,v 1.8 2002/03/22 21:23:02 chris Exp $
 *
 * SEGA Dreamcast support using Kallistios (http://cadcdev.sourceforge.net)
 * (c) 2002 Christian Groessler (chris@groessler.org)
 */

#include <stdarg.h>
#include <stdlib.h>
#include <kos.h>
#include "atari.h"
#include "config.h"
#include "input.h"
#include "rt-config.h"
#include "colours.h"
#include "ui.h"
#ifdef SOUND
#include "sound.h"
#include "pokeysnd.h"
#endif

#define min(x, y)  ((x)>(y)?(y):(x))

/*#define DEBUG*/
#define ALWAYS_CONTROLLER_KEYS

static void calc_palette(void);
#ifdef SOUND
static void dc_sound_init(void);
#endif

static uint8 mcont[4];
static uint8 mkb;
static int num_cont;   /* # of controllers found */

#ifdef DC_RAMDISK
extern uint8 romdisk[];
#define DC_ROMDISK romdisk
#else
#define DC_ROMDISK ROMDISK_NONE
#endif

void Atari_Initialise(int *argc, char *argv[])
{
	static int fc = TRUE;

	if (! fc) return; else fc = FALSE;

	/* Bother us with output only if something died */
#ifndef DEBUG
	dbglog_set_level(DBG_DEAD);
#endif
	
        /* Set the video mode */
#ifdef DEBUG
	vid_set_mode(DM_640x480, PM_RGB565);  /* debug mode assumes VGA monitor */
#else
	vid_set_mode(DM_320x240, PM_RGB565);  /* non-debug assumes TV */
#endif
	calc_palette();
}

#ifdef SUPPORTS_ATARI_CONFIGINIT

void Atari_ConfigInit(void)  /* not currently used */
{
	strcpy(atari_osa_filename, "/cd/atariosa.rom");
	strcpy(atari_osb_filename, "/cd/atariosb.rom");
	strcpy(atari_xlxe_filename, "/cd/atarixl.rom");
	strcpy(atari_basic_filename, "/cd/ataribas.rom");
	atari_5200_filename[0] = '/cd/5200.rom';
	strcpy(atari_disk_dirs[0], "/cd/disks");
	disk_directories = 1;
	strcpy(atari_rom_dir,"/cd/cartridges");
	atari_h1_dir[0] = '\0';
	atari_h2_dir[0] = '\0';
	atari_h3_dir[0] = '\0';
	atari_h4_dir[0] = '\0';
	strcpy(atari_exe_dir,"/cd/files");
	strcpy(atari_state_dir,"\0");
}

#endif

int Atari_Exit(int run_monitor)
{
#if 0
	if (run_monitor)
		return monitor();
#endif
	return FALSE;
}

static int consol_keys(void)
{
	cont_cond_t cond;

	if (cont_get_cond(mcont[0], &cond)) {
#ifdef DEBUG
		printf("consol_keys: error getting controller status\n");
#endif
		return(AKEY_NONE);
	}

	if (! (cond.buttons & CONT_START)) {
		if (ui_is_active)
			arch_reboot();
		else {
			if (! (cond.buttons & CONT_X))
				return(AKEY_COLDSTART);
			else
				return(AKEY_WARMSTART);
		}
	}

	if (! (cond.buttons & CONT_B)) {
		key_consol &= ~CONSOL_START;
	}
	else {
		key_consol |= CONSOL_START;
	}

	if (! (cond.buttons & CONT_Y)) {
		key_consol &= ~CONSOL_SELECT;
	}
	else {
		key_consol |= CONSOL_SELECT;
	}

	if (! (cond.buttons & CONT_X)) {
		key_consol &= ~CONSOL_OPTION;
	}
	else {
		key_consol |= CONSOL_OPTION;
	}
	return(AKEY_NONE);
}

/*
 * do some basic keyboard emulation for the controller,
 * so that the emulator menu can be used without keyboard
 * (basically for selecting bin files/disk images)
 */
static int controller_kb(void)
{
	static int prev_up = FALSE, prev_down = FALSE, prev_a = FALSE,
		prev_r = FALSE, prev_left = FALSE, prev_right = FALSE;
	cont_cond_t cond;

	if (! num_cont) return(AKEY_NONE);  /* no controller present */

	if (cont_get_cond(mcont[0], &cond)) {
#ifdef DEBUG
		printf("controller_kb: error getting controller status\n");
#endif
		return(AKEY_NONE);
	}

	if (cond.ltrig > 250) {
		return(AKEY_UI);
	}

	if (ui_is_active) {
		if (! (cond.buttons & CONT_DPAD_UP)) {
			if (! prev_up) {
				prev_up = 1;
				return(AKEY_UP);
			}
		}
		else {
			prev_up = FALSE;
		}
		if (! (cond.buttons & CONT_DPAD_DOWN)) {
			if (! prev_down) {
				prev_down = TRUE;
				return(AKEY_DOWN);
			}
		}
		else {
			prev_down = FALSE;
		}

		if (! (cond.buttons & CONT_DPAD_LEFT)) {
			if (! prev_left) {
				prev_left = TRUE;
				return(AKEY_LEFT);
			}
		}
		else {
			prev_left = FALSE;
		}

		if (! (cond.buttons & CONT_DPAD_RIGHT)) {
			if (! prev_right) {
				prev_right = TRUE;
				return(AKEY_RIGHT);
			}
		}
		else {
			prev_right = FALSE;
		}

		if (! (cond.buttons & CONT_A)) {
			if (! prev_a) {
				prev_a = TRUE;
				return(AKEY_RETURN);
			}
		}
		else {
			prev_a = FALSE;
		}

		if (cond.rtrig > 250) {
			if (! prev_r) {
				prev_r = TRUE;
				return(AKEY_ESCAPE);
			}
		}
		else {
			prev_r = FALSE;
		}
	}
	return(AKEY_NONE);
}

int Atari_Keyboard(void)
{
	int keycode;

	if (num_cont && (keycode = consol_keys()) != AKEY_NONE) return(keycode);

#ifdef ALWAYS_CONTROLLER_KEYS
	keycode = controller_kb();
	if (keycode != AKEY_NONE) return(keycode);
#else
	if (! mkb) {    /* if no keyboard present do some basic emulation with controller */
		return(controller_kb());
	}
#endif

	keycode = kbd_get_key();
	if (keycode == -1) return(AKEY_NONE);

#ifdef DEBUG
	printf("DC key: %04X (%c)\n", keycode, keycode > 32 && keycode < 127 ? keycode : '.');
#endif

#if 0 /* later.. */
	/* OPTION / SELECT / START keys */
	key_consol = CONSOL_NONE;
	switch (keycode) {
	case 0x3b00:
		key_consol &= (~CONSOL_OPTION);
		return(AKEY_NONE);
	case 0x3c00:
		key_consol &= (~CONSOL_SELECT);
		return(AKEY_NONE);
	case 0x3d00:
		key_consol &= (~CONSOL_START);
		return(AKEY_NONE);
	}
#endif

	switch (keycode) {
	case 0x1b:  /* ESC */
		keycode = AKEY_ESCAPE;
		break;
	case '`':
		keycode = AKEY_CAPSTOGGLE;
		break;
	case '!':
		keycode = AKEY_EXCLAMATION;
		break;
	case '"':
		keycode = AKEY_DBLQUOTE;
		break;
	case '#':
		keycode = AKEY_HASH;
		break;
	case '$':
		keycode = AKEY_DOLLAR;
		break;
	case '%':
		keycode = AKEY_PERCENT;
		break;
	case '&':
		keycode = AKEY_AMPERSAND;
		break;
	case '\'':
		keycode = AKEY_QUOTE;
		break;
	case '@':
		keycode = AKEY_AT;
		break;
	case '(':
		keycode = AKEY_PARENLEFT;
		break;
	case ')':
		keycode = AKEY_PARENRIGHT;
		break;
	case '[':
		keycode = AKEY_BRACKETLEFT;
		break;
	case ']':
		keycode = AKEY_BRACKETRIGHT;
		break;
	case '<':
		keycode = AKEY_LESS;
		break;
	case '>':
		keycode = AKEY_GREATER;
		break;
	case '=':
		keycode = AKEY_EQUAL;
		break;
	case '?':
		keycode = AKEY_QUESTION;
		break;
	case '-':
		keycode = AKEY_MINUS;
		break;
	case '+':
		keycode = AKEY_PLUS;
		break;
	case '*':
		keycode = AKEY_ASTERISK;
		break;
	case '/':
		keycode = AKEY_SLASH;
		break;
	case ':':
		keycode = AKEY_COLON;
		break;
	case ';':
		keycode = AKEY_SEMICOLON;
		break;
	case ',':
		keycode = AKEY_COMMA;
		break;
	case '.':
		keycode = AKEY_FULLSTOP;
		break;
	case '_':
		keycode = AKEY_UNDERSCORE;
		break;
	case '^':
		keycode = AKEY_CIRCUMFLEX;
		break;
	case '\\':
		keycode = AKEY_BACKSLASH;
		break;
	case '|':
		keycode = AKEY_BAR;
		break;
	case ' ':
		keycode = AKEY_SPACE;
		break;
	case '0':
		keycode = AKEY_0;
		break;
	case '1':
		keycode = AKEY_1;
		break;
	case '2':
		keycode = AKEY_2;
		break;
	case '3':
		keycode = AKEY_3;
		break;
	case '4':
		keycode = AKEY_4;
		break;
	case '5':
		keycode = AKEY_5;
		break;
	case '6':
		keycode = AKEY_6;
		break;
	case '7':
		keycode = AKEY_7;
		break;
	case '8':
		keycode = AKEY_8;
		break;
	case '9':
		keycode = AKEY_9;
		break;
	case 'a':
		keycode = AKEY_a;
		break;
	case 'b':
		keycode = AKEY_b;
		break;
	case 'c':
		keycode = AKEY_c;
		break;
	case 'd':
		keycode = AKEY_d;
		break;
	case 'e':
		keycode = AKEY_e;
		break;
	case 'f':
		keycode = AKEY_f;
		break;
	case 'g':
		keycode = AKEY_g;
		break;
	case 'h':
		keycode = AKEY_h;
		break;
	case 'i':
		keycode = AKEY_i;
		break;
	case 'j':
		keycode = AKEY_j;
		break;
	case 'k':
		keycode = AKEY_k;
		break;
	case 'l':
		keycode = AKEY_l;
		break;
	case 'm':
		keycode = AKEY_m;
		break;
	case 'n':
		keycode = AKEY_n;
		break;
	case 'o':
		keycode = AKEY_o;
		break;
	case 'p':
		keycode = AKEY_p;
		break;
	case 'q':
		keycode = AKEY_q;
		break;
	case 'r':
		keycode = AKEY_r;
		break;
	case 's':
		keycode = AKEY_s;
		break;
	case 't':
		keycode = AKEY_t;
		break;
	case 'u':
		keycode = AKEY_u;
		break;
	case 'v':
		keycode = AKEY_v;
		break;
	case 'w':
		keycode = AKEY_w;
		break;
	case 'x':
		keycode = AKEY_x;
		break;
	case 'y':
		keycode = AKEY_y;
		break;
	case 'z':
		keycode = AKEY_z;
		break;
	case 'A':
		keycode = AKEY_A;
		break;
	case 'B':
		keycode = AKEY_B;
		break;
	case 'C':
		keycode = AKEY_C;
		break;
	case 'D':
		keycode = AKEY_D;
		break;
	case 'E':
		keycode = AKEY_E;
		break;
	case 'F':
		keycode = AKEY_F;
		break;
	case 'G':
		keycode = AKEY_G;
		break;
	case 'H':
		keycode = AKEY_H;
		break;
	case 'I':
		keycode = AKEY_I;
		break;
	case 'J':
		keycode = AKEY_J;
		break;
	case 'K':
		keycode = AKEY_K;
		break;
	case 'L':
		keycode = AKEY_L;
		break;
	case 'M':
		keycode = AKEY_M;
		break;
	case 'N':
		keycode = AKEY_N;
		break;
	case 'O':
		keycode = AKEY_O;
		break;
	case 'P':
		keycode = AKEY_P;
		break;
	case 'Q':
		keycode = AKEY_Q;
		break;
	case 'R':
		keycode = AKEY_R;
		break;
	case 'S':
		keycode = AKEY_S;
		break;
	case 'T':
		keycode = AKEY_T;
		break;
	case 'U':
		keycode = AKEY_U;
		break;
	case 'V':
		keycode = AKEY_V;
		break;
	case 'W':
		keycode = AKEY_W;
		break;
	case 'X':
		keycode = AKEY_X;
		break;
	case 'Y':
		keycode = AKEY_Y;
		break;
	case 'Z':
		keycode = AKEY_Z;
		break;

	case 0x3a00:  /* F1 */
		keycode = AKEY_UI;
		break;

	case 0x3e00:  /* F5 */
		keycode = AKEY_COLDSTART;
		break;

	case 0x4500:  /* F12 */
		arch_reboot();

	/* cursor keys */
	case 0x5200:
		keycode = AKEY_UP;
		break;
	case 0x5100:
		keycode = AKEY_DOWN;
		break;
	case 0x5000:
		keycode = AKEY_LEFT;
		break;
	case 0x4f00:
		keycode = AKEY_RIGHT;
		break;

	case 0x4a00:  /* Home */
		keycode = AKEY_CLEAR;
		break;
	case 0x4d00:  /* End */
		keycode = AKEY_HELP;
		break;
	case 0x4800:  /* Break/Pause */
		keycode = AKEY_BREAK;
		break;

	case 0x4c00:  /* Del */
		if (key_shift)
			keycode = AKEY_DELETE_LINE;
		else
			keycode |= AKEY_DELETE_CHAR;
		break;
	case 0x4900:  /* Ins */
		if (key_shift)
			keycode = AKEY_INSERT_LINE;
		else
			keycode |= AKEY_INSERT_CHAR;
		break;

	case 127:
	case 8:
		keycode = AKEY_BACKSPACE;
		break;
	case 13:
	case 10:
		keycode = AKEY_RETURN;
		break;
	default:
		keycode = AKEY_NONE;
		break;
	}
	return keycode;
}

int Atari_PORT(int num)
{
	cont_cond_t cond;
	int retval = 0xff;
	int cix = 0;

	if (! num_cont) return(retval);  /* no controller present */

	if (num) {
	  if (machine_type != MACHINE_OSA && machine_type != MACHINE_OSB) return(retval);
	  cix = 2;   /* js #2 and #3 */
	}

	if (cont_get_cond(mcont[cix], &cond)) {    /* first controller */
#ifdef DEBUG
		printf("Atari_TRIG: error getting controller status\n");
#endif
		return(0xff);
	}
	if (! (cond.buttons & CONT_DPAD_UP)) {
		retval &= 0xfe;
	}
	if (! (cond.buttons & CONT_DPAD_DOWN)) {
		retval &= 0xfd;
	}
	if (! (cond.buttons & CONT_DPAD_LEFT)) {
		retval &= 0xfb;
	}
	if (! (cond.buttons & CONT_DPAD_RIGHT)) {
		retval &= 0xf7;
	}
	if (num_cont > 1 + cix) {
		if (cont_get_cond(mcont[cix+1], &cond)) {    /* first controller */
#ifdef DEBUG
			printf("Atari_TRIG: error getting controller status (#1)\n");
#endif
			return(retval);
		}
		if (! (cond.buttons & CONT_DPAD_UP)) {
			retval &= 0xef;
		}
		if (! (cond.buttons & CONT_DPAD_DOWN)) {
			retval &= 0xdf;
		}
		if (! (cond.buttons & CONT_DPAD_LEFT)) {
			retval &= 0xbf;
		}
		if (! (cond.buttons & CONT_DPAD_RIGHT)) {
			retval &= 0x7f;
		}
	}

	return(retval);
}

int Atari_TRIG(int num)
{
	cont_cond_t cond;

	if (num + 1 > num_cont) return(1);  /* no controller present */
	if (machine_type != MACHINE_OSA && machine_type != MACHINE_OSB && num > 1) return(1);

	if (cont_get_cond(mcont[num], &cond)) {
#ifdef DEBUG
		printf("Atari_TRIG: error getting controller status (%d)\n", num);
#endif
		return(1);
	}
	if (! (cond.buttons & CONT_A)) {
		return(0);
	}

	return(1);
}

static uint16 mypal[256];

static void calc_palette(void)
{
	int i;
	int rgb;
	int red;
	int green;
	int blue;
	uint16 pv;

	for (i = 0; i < 256; i++) {
		rgb = colortable[i];

		red = (rgb & 0x00ff0000) >> 16;
		green = (rgb & 0x0000ff00) >> 8;
		blue = (rgb & 0x000000ff) ;

		pv = ((red >> 3) << 11) |
			((green >> 2) << 5) |
			(blue >> 3);

		mypal[i] = pv;
	}
}


void Atari_DisplayScreen(UBYTE * screen)
{
	int i, j;
	uint16 *vram = vram_s;

	vram += (vid_mode->height - 240) / 2 * vid_mode->width;

	for (i=0; i<240; i++) {         /* y */
		screen += (ATARI_WIDTH - 320) / 2;
		vram += (vid_mode->width - 320) / 2;
		for (j=0; j<320; j++) { /* x */
			*vram++ = mypal[*screen++];
		}
		vram += (vid_mode->width - 320) / 2;
		screen += (ATARI_WIDTH - 320) / 2;
	}
}

static void dc_controller_init(void)
{
	int p, u;

	for (p=0; p<MAPLE_PORT_COUNT; p++) {
		for (u=0; u<MAPLE_UNIT_COUNT; u++) {
			if (maple_device_func(p, u) & MAPLE_FUNC_CONTROLLER) {
				mcont[num_cont++] = maple_addr(p, u);
#ifdef DEBUG
				printf("dc_controller_init: found controller %d\n", mcont[num_cont-1]);
#endif
			}
		}
	}

#if 0
	mcont[0] = maple_first_controller();
	if (mcont[0]) num_cont++;
#endif
	mkb = maple_first_kb();

#ifdef DEBUG
	printf("dc_controller_init: controller%sfound, keyboard%sfound\n", 
	       mcont ? " " : " not ", mkb ? " " : " not ");
#endif
	return;
}

static void cls(int xres, int yres)
{
	int x, y;
	for(x = 0; x < xres; x++)
		for(y = 0; y < yres; y++)
			vram_s[xres*y + x] = 0;
}

static void wait_a(void)
{
	cont_cond_t cond;

	do {
		if (cont_get_cond(mcont[0], &cond)) {
			return;
		}

	} while (cond.buttons & CONT_A);
}


int main(int argc, char **argv)
{
	/* Do initial setup */	
	kos_init_all(BASIC_ENABLE, DC_ROMDISK);

	/* initialise Atari800 core */
	Atari800_Initialise(&argc, argv);

	/* initialize dc controllers */
	dc_controller_init();

#ifdef SOUND
	/* initialize sound */
	dc_sound_init();
#endif

#ifdef DEBUG
        printf("\nFor Tobias & Dominik\n");
        printf("--------------------\n");
#endif

	/* main loop */
	while(TRUE)
	{
		static int test_val = 0;
		int keycode;

		keycode = Atari_Keyboard();

		switch (keycode) {
		case AKEY_COLDSTART:
			Coldstart();
			break;
		case AKEY_WARMSTART:
			Warmstart();
			break;
		case AKEY_EXIT:
			Atari800_Exit(FALSE);
			arch_reboot();
		case AKEY_UI:
#ifdef SOUND
			Sound_Pause();
#endif
			ui((UBYTE *)atari_screen);
#ifdef SOUND
			Sound_Continue();
#endif
			break;
#if 0
		case AKEY_SCREENSHOT:
			Save_PCX_file(FALSE, Find_PCX_name());
			break;
		case AKEY_SCREENSHOT_INTERLACE:
			Save_PCX_file(TRUE, Find_PCX_name());
			break;
#endif
		case AKEY_BREAK:
			key_break = 1;
			break;
		default:
			key_break = 0;
			key_code = keycode;
			break;
		}

		if (++test_val == refresh_rate)
		{
			Atari800_Frame(EMULATE_FULL);
#ifndef DONT_SYNC_WITH_HOST
			atari_sync(); /* here seems to be the best place to sync */
#endif
			Atari_DisplayScreen((UBYTE *) atari_screen);
			test_val = 0;
		}
		else
		{
#ifdef VERY_SLOW
			Atari800_Frame(EMULATE_BASIC);
#else	/* VERY_SLOW */
			Atari800_Frame(EMULATE_NO_SCREEN);
#ifndef DONT_SYNC_WITH_HOST
			atari_sync();
#endif
#endif	/* VERY_SLOW */
		}
	}
}

int sscanf(const char *str, const char *format, ...)
{
	va_list args;
	int *res;

	if (strcmp(format, "%d")) return(0);   /* sanity check, we only support "%d" */

	va_start(args, format);
	res = va_arg(args, int *);
	*res = atoi(str);
        va_end(args);
	return(1);
}

static char mytmpnam[] = "tempname.oef";
char *tmpnam(char *space)
{
	if (space) {
		strcpy(space, mytmpnam);
		return(space);
	}
	else {
		return(mytmpnam);
	}
}
int unlink(const char *x) { return 0; }
int remove(const char *x) { return 0; } 
void dc_perror(const char *x) { }
void (*signal(int sig, void (*func)(int)))(int) { return (void *) 0; }

void SaveUBYTE( UBYTE *data, int num )
{
	return;
}
void SaveINT( int *data, int num ) 
{
	return;
}
void SaveUWORD( UWORD *data, int num )
{
	return;
}
void ReadUBYTE( UBYTE *data, int num )
{
	return;
}
void ReadINT( int *data, int num ) 
{
	return;
}
void ReadUWORD( UWORD *data, int num )
{
	return;
}
int SaveAtariState( char *filename, const char *mode, UBYTE SaveVerbose )
{
	return(FALSE);
}
int ReadAtariState( char *filename, const char *mode )
{
	return(FALSE);
}

#ifdef SOUND

static int sound_enabled = TRUE;        /* sound: on or off */

#define FRAGSIZE        15		/* 1<<FRAGSIZE is size of sound buffer */

#define B_P_R 1024    /* bytes per "round" */
#define dsprate 22050
/*#define dsprate 44100*/
/*#define dsprate 15960*/
static unsigned char *audio_buffer;
static unsigned short *word_buffer;
int auix;

extern int reallen;
/*int poll_call;*/

static void *snd_callback(int size)
{
	uint16 *hlps = word_buffer;
	uint8 *hlpc = audio_buffer;
	int i;

	/*
	printf("snd_callback called (%d poll, %d bytes wanted)\n", poll_call, size);
	poll_call = 0;
	*/

	if (size > (1 << FRAGSIZE)) {  /* don't overrun our buffer */
#ifdef DEBUG
		printf("snd_callback: we lose\n");
#endif
		size = 1 << FRAGSIZE;
        }

#ifdef NEW_SOUND
	if (size > auix) {
		/* we need to generate the missing sound data */
		Pokey_process(hlpc + auix, size - auix);
		printf("snd_callback: generated %d\n", size - auix);
	}
#else
	Pokey_process(hlpc,size);
#endif

	if (! sound_enabled) {
		for (i=0; i<size; i++) {
			*(hlps + i) = 0;
		}
	}
	else {
		for (i=0; i<size; i++) {
			*(hlps + i) = *hlpc++ << 8;   /* convert 8 -> 16 */
		}
	}

#ifdef NEW_SOUND
	if (auix > size) {
		memcpy(audio_buffer, audio_buffer + size, auix - size);
		auix -= size;
	}
	else {
		auix = 0;
	}
#endif
	reallen = size;
	return(hlps);
}

void dc_sound_init(void)
{
        snd_stream_init(snd_callback);

	if (audio_buffer == NULL) {
		audio_buffer = malloc(1 << FRAGSIZE);
		word_buffer = malloc((1 << FRAGSIZE) * 2);
		auix = 0;   /* initialize buffer index */
	}

	Pokey_sound_init(FREQ_17_EXACT, dsprate, 1);

#ifdef STEREO
#warning STEREO not implemented
#endif
	snd_stream_start(dsprate, 0);
}

void Sound_Pause(void)
{
	if (sound_enabled) {
		sound_enabled = FALSE;
		snd_stream_stop();
	}
}

void Sound_Continue(void)
{
	if (! sound_enabled) {
		sound_enabled = TRUE;
		snd_stream_start(dsprate, 0);
	}
}

void Sound_Exit(void)
{
#ifdef DEBUG
        printf("Sound_Exit called\n");
#endif
}

void Sound_Update(void)
{
	/*int len;*/

	if (! sound_enabled)
		return;

#ifdef NEW_SOUND
	if (auix < (1 << FRAGSIZE)) {
		len = min(B_P_R, (1 << FRAGSIZE) - auix);
		if (len) {
			Pokey_process(audio_buffer + auix, len);
			auix += len;
		}
	}
#endif

	/*poll_call++;*/
	snd_stream_poll();
}

#endif	/* #ifdef SOUND */
